# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from .policy_id import PolicyId
from .rule import Rule
from .trigger import Trigger


class ApprovalPolicyRequest(pydantic.BaseModel):
    trigger: Trigger
    rule: Rule
    upstream_policy_id: PolicyId = pydantic.Field(
        alias="upstreamPolicyId", description=("Use 'root' if no upstreamPolicyId is intended to be set.\n")
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
