# This file was auto-generated by Fern from our API Definition.

from .types import Attachments, EmailOcrRequest, OcrMailbox, OCRResponse

__all__ = ["Attachments", "EmailOcrRequest", "OCRResponse", "OcrMailbox"]
