# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions

from .approver_rule import ApproverRule


class Rule_Approver(ApproverRule):
    type: typing_extensions.Literal["approver"]

    class Config:
        frozen = True


Rule = typing_extensions.Annotated[typing.Union[Rule_Approver], pydantic.Field(discriminator="type")]
